/*
 * Synopsis:
 *	int disrfcs(int stream, size_t *nchars, size_t achars, char *value)
 *
 *	Gets a Data-is-Strings character string from <stream> and converts it
 *	into a counted string, puts the string into <value>, a pre-allocated
 *	string, <achars> long, and puts the count into *<nchars>.  The character
 *	string in <stream> consists of an unsigned integer, followed by a number
 *	of characters determined by the unsigned integer.
 *
 *	Disrfcs returns DIS_SUCCESS if everything works well.  It returns an
 *	error code otherwise.  In case of an error, the <stream> character
 *	pointer is reset, making it possible to retry with some other conversion
 *	strategy, and <nchars> is set to 0.
 */
#include <pbs_config.h>   /* the master config generated by configure */

#include <assert.h>
#include <stddef.h>
#include <stdlib.h>

#include "dis.h"
#include "dis_.h"

int disrfcs(stream, nchars, achars, value)
    int			stream;
    size_t		*nchars;
    size_t		achars;
    char		*value;
    {
	int		locret;
	int		negate;
	unsigned	count = 0;

	assert(nchars != NULL);
	assert(value != NULL);
	assert(dis_gets != NULL);
	assert(disr_commit != NULL);

	locret = disrsi_(stream, &negate, &count, 1);
	if (locret == DIS_SUCCESS) {
		if (negate)
		        locret = DIS_BADSIGN;
		else if ((*nchars = count) > achars)
		        locret = DIS_OVERFLOW;
		else if ((*dis_gets)(stream, value, *nchars) != *nchars)
		        locret = DIS_PROTO;
	}
	locret = (*disr_commit)(stream, locret == DIS_SUCCESS) ?
	    DIS_NOCOMMIT : locret;
	if (locret != DIS_SUCCESS)
	        *nchars = 0;
	return (locret);
}
